#include <EEPROM.h>
#include <Servo.h>
#include <Bounce2.h>
#include <Adafruit_SSD1306.h>
#include "MPU6050.h"


// LCD reset PIN. We do not use it.
#define LCD_RESET_PIN     -1

// Button PINs.
#define BUTTON_RIGHT_PIN  2
#define BUTTON_OK_PIN     3
#define BUTTON_LEFT_PIN   4

// ESC PIN.
#define ESC_PIN           5

// Piezo speaker PIN.
#define BEEPER_PIN        6

// LED PIN. By default use build-in LED on PIN 13.
#define LED_PIN           LED_BUILTIN

// Settings EEPROM address.
#define SETTINGS_ADDR 0

// MPU6050 I2C address.
#define MPU_I2C_ADDR  0x68

// LCD I2C address.
#define LCD_I2C_ADDR  0x3C


// Vibrometer settings.
struct Settings
{
  uint16_t  minPulse; // Munimum pulse send to ESC to arm it.
  uint16_t  runPulse; // Pulse send to ESC to run motor during measure.
  uint8_t   runTime;  // Measuring time in seconds.
};

// Buttons.
enum Button
{
  BtnLeft,
  BtnOK,
  BtnRight
};

// Vibrometer settings.
Settings settings;


//--------------------------------------------------------------------------------------
// BEEPER
// Motor start tone.
#define TONE_START    3038
// Motor stop tone.
#define TONE_STOP     3400
// Filed tone.
#define TONE_FAILED   3830

void beep(uint16_t note)
{
  uint32_t elapsedTime = 0;

  while (elapsedTime < 500000)
  {
    digitalWrite(BEEPER_PIN, HIGH);
    delayMicroseconds(note / 2);
    digitalWrite(BEEPER_PIN, LOW);
    delayMicroseconds(note / 2);
    elapsedTime = elapsedTime + note;
  }
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// LED
uint8_t ledState = LOW; // Current LED state.

// Turn LED on.
void ledOn()
{
  if (ledState == LOW)
  {
    ledState = HIGH;
    digitalWrite(LED_PIN, ledState);
  }
}

// Turn LED off.
void ledOff()
{
  if (ledState == HIGH)
  {
    ledState = LOW;
    digitalWrite(LED_PIN, ledState);
  }
}

// Invert LED (on if off, off if on).
void ledInvert()
{
  if (ledState == LOW)
    ledState = HIGH;
  else
    ledState = LOW;
  digitalWrite(LED_PIN, ledState);
}

void ledBlink()
{
  ledInvert();
  delay(500);
  ledInvert();
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// BUTTONS
#define BUTTON_DEBOUNCE_INTERVAL  5

Bounce btnLeft;
Bounce btnOk;
Bounce btnRight;

// Return true if Left buttons is pressed.
bool leftPressed()
{
  return (btnLeft.update() && btnLeft.read() == HIGH);
}

// Return true if OK button is pressed.
bool okPressed()
{
  return (btnOk.update() && btnOk.read() == HIGH);
}

// Return true if Right button is pressed.
bool rightPressed()
{
  return (btnRight.update() && btnRight.read() == HIGH);
}

// Wait when burron is pressed and return which one.
Button getButton()
{
  while (1)
  {
    if (leftPressed())
      return BtnLeft;

    if (okPressed())
      return BtnOK;

    if (rightPressed())
      return BtnRight;
  }
}

// Checks if given button is pressed.
bool isPressed(Button btn)
{
  switch (btn)
  {
    case BtnLeft:
      return leftPressed();

    case BtnOK:
      return okPressed();

    case BtnRight:
      return rightPressed();

    default:
      return false;
  }
}

// Wait for press on given button.
void waitButton(Button btn)
{
  while (!isPressed(btn)) ;
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// LCD
#define FONT_WIDTH  6
#define FONT_HEIGHT 8

Adafruit_SSD1306 lcd(LCD_RESET_PIN);

uint8_t getTextWidth(String s)
{
  return (s.length() * FONT_WIDTH);
}

uint8_t getTextCenter(String s)
{
  return (SSD1306_LCDWIDTH - getTextWidth(s)) / 2;
}

// Return next line y position.
uint8_t printCenter(String s, uint8_t y)
{
  lcd.setCursor(getTextCenter(s), y);
  lcd.print(s);
  return (y + FONT_HEIGHT + 2);
}

// Return next line y position.
uint8_t showTitle(String s)
{
  lcd.clearDisplay();
  lcd.setTextColor(WHITE);

  uint8_t y = printCenter(s, 0);
  lcd.drawFastHLine(0, y, SSD1306_LCDWIDTH, WHITE);

  return (y + 4);
}

// Return next line y position.
uint8_t showMenuItem(String s, uint8_t y, bool selected)
{
  if (selected)
    lcd.setTextColor(BLACK, WHITE);
  y = printCenter(s, y);
  if (selected)
    lcd.setTextColor(WHITE);

  return y;
}

// Return next line y position.
uint8_t drawProgress(uint8_t y, uint8_t pos, uint8_t max)
{
  uint8_t width = map(pos, 0, max, 0, SSD1306_LCDWIDTH);

  lcd.drawFastHLine(0, y, width, WHITE);
  y++;
  lcd.drawFastHLine(0, y, width, WHITE);
  y++;

  return y;
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// MPU6050

// Amount of readings used to average, make it higher to get more precision but sketch will be slower (default: 1000)
#define BUFFER_SIZE     500
// Acelerometer error allowed, make it lower to get more precision, but sketch may not converge
#define ACCEL_DEADZONE  8
// Gyro error allowed, make it lower to get more precision, but sketch may not converge
#define GYRO_DEADZONE   1
// Z-axis offset.
#define Z_ACC_CONST     16384
// Maximum calibration attempts.
#define MAX_CNT         12

MPU6050 mpu(MPU_I2C_ADDR);

String intToStr(int16_t v)
{
  String s = String(v);
  while (s.length() < 6)
    s = String(" " + s);
  return s;
}

void showCalibrationProcess(uint8_t pos)
{
  uint8_t y = showTitle("CALIBRATING");
  y += 5;
  y = printCenter("Calibrating...", y);
  y += 5;
  y = printCenter("Keep sensor still", y);
  y += 5;
  drawProgress(y, pos, MAX_CNT);
  lcd.display();
}

// Read AVG from accelerometers.
void meanSensors(int16_t& mean_ax, int16_t& mean_ay, int16_t& mean_az)
{
  int16_t ax = 0;
  int16_t ay = 0;
  int16_t az = 0;
  int16_t gx = 0;
  int16_t gy = 0;
  int16_t gz = 0;

  long i = 0;
  long buff_ax = 0;
  long buff_ay = 0;
  long buff_az = 0;

  while (i < (BUFFER_SIZE + 101))
  {
    if (i % 20 == 0)
      ledInvert();
    
    mpu.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);

    if (i > 100 && i <= (BUFFER_SIZE + 100))
    {
      buff_ax += ax;
      buff_ay += ay;
      buff_az += az;
    }

    if (i == (BUFFER_SIZE + 100))
    {
      mean_ax = buff_ax / BUFFER_SIZE;
      mean_ay = buff_ay / BUFFER_SIZE;
      mean_az = buff_az / BUFFER_SIZE;
    }

    i++;

    delay(2);
  }
}

// Calibrate accel. Return true if calibration was success. False if not.
bool calibrate()
{
  while (1)
  {
    ledOn();
    
    uint8_t y = showTitle("CALIBRATION");
    y += 5;
    y = printCenter("Ready for calibration", y);
    y += 5;
    y = printCenter("Keep sensor still", y);
    lcd.display();
    delay(2000); // Allow user to leave sensor alone.

    // Initialization.
    uint8_t cnt = 0;
    showCalibrationProcess(cnt);
    cnt++;

    mpu.setXAccelOffset(0);
    mpu.setYAccelOffset(0);
    mpu.setZAccelOffset(0);

    int16_t mean_ax = 0;
    int16_t mean_ay = 0;
    int16_t mean_az = 0;

    // First time read.
    meanSensors(mean_ax, mean_ay, mean_az);

    // First offsets.
    int16_t ax_offset = -mean_ax / 8;
    int16_t ay_offset = -mean_ay / 8;
    int16_t az_offset = (Z_ACC_CONST - mean_az) / 8;

    // Trying to calibrate.
    while (1)
    {
      showCalibrationProcess(cnt);
      cnt++;
      if (cnt > MAX_CNT)
        break;

      int ready = 0;

      mpu.setXAccelOffset(ax_offset);
      mpu.setYAccelOffset(ay_offset);
      mpu.setZAccelOffset(az_offset);

      meanSensors(mean_ax, mean_ay, mean_az);

      if (abs(mean_ax) <= ACCEL_DEADZONE)
        ready++;
      else
        ax_offset = ax_offset - mean_ax / ACCEL_DEADZONE;
      
      if (abs(mean_ay) <= ACCEL_DEADZONE)
        ready++;
      else
        ay_offset = ay_offset - mean_ay / ACCEL_DEADZONE;
      
      if (abs(Z_ACC_CONST - mean_az) <= ACCEL_DEADZONE)
        ready++;
      else
        az_offset = az_offset + (Z_ACC_CONST - mean_az) / ACCEL_DEADZONE;

      if (ready == 3)
        break;
    }

    // Read calibrated values.
    meanSensors(mean_ax, mean_ay, mean_az);

    ledOff();

    // if calibration was not success
    if (mean_ax < -10 || mean_ax > 10 || mean_ay < -10 || mean_ay > 10 || mean_az < 16374 || mean_az > 16394)
    {
      y = showTitle("CALIBRATED");
      y += 5;
      lcd.setTextColor(BLACK, WHITE);
      y = printCenter(" Calibration failed ", y);
      lcd.setTextColor(WHITE);
      y += 5;
      y = printCenter("Press OK to recal", y);
      printCenter("Press Left to exit", y);
      lcd.display();

      beep(TONE_FAILED);

      // We can redo or exit.
      // "OK" - redo
      // "Left" - Exit.
      Button btn;
      while ((btn = getButton()) == BtnRight) ;
      if (btn == BtnLeft)
        return false;
      else
        continue;
    }
    else
      return true;
  }
}

// Vibration measuring.
void measure()
{
  while (1)
  {
    uint8_t y;

    // Check that MPU is connection.
    if (!mpu.testConnection())
    {
      y = showTitle("MEASURE VIBRATION");
      y += 5;
      lcd.setTextColor(BLACK, WHITE);
      y = printCenter(" MPU FAILED ", y);
      lcd.setTextColor(WHITE);
      y += 5;
      printCenter("Press OK to exit", y);
      lcd.display();

      beep(TONE_FAILED);

      waitButton(BtnOK);

      return;;
    }

    // Calibrate accels.
    if (!calibrate())
      return;

    float prev_av_ax, prev_av_ay, prev_av_az;// = 0;

    Button btn;

    // Start measuring.
    while (1)
    {
      // Notify about motor spin.
      y = showTitle("MEASURE VIBRATION");
      y += 5;
      y = printCenter("Preparing for measure", y);
      lcd.display();
      delay(2000);

      y = showTitle("MEASURING");
      y += 5;
      lcd.setTextColor(BLACK, WHITE);
      y = printCenter(" BE CAREFULL! ", y);
      lcd.setTextColor(WHITE);
      y += 5;
      printCenter("Starting motor", y);
      lcd.display();
      delay(500);

      motorStart(); // Start here.

      // Notification about running morot.
      y = showTitle("MEASURING");
      y += 5;
      lcd.setTextColor(BLACK, WHITE);
      y = printCenter(" BE CAREFULL! ", y);
      lcd.setTextColor(WHITE);
      y += 5;
      printCenter("Motor is running", y);
      lcd.display();

      // Initialization.
      int16_t ax = 0;
      int16_t ay = 0;
      int16_t az = 0;
      int16_t gx = 0;
      int16_t gy = 0;
      int16_t gz = 0;

      float av_ax = 0;
      float av_ay = 0;
      float av_az = 0;

      long time = micros();
      long m_cnt = 0;

      // Measuring.
      while (1)
      {
        if (m_cnt % 100 == 0)
          ledInvert();
        
        mpu.getMotion6(&ax, &ay, &az, &gx, &gy, &gz);

        av_ax = (float)(av_ax * m_cnt + abs(ax)) / (m_cnt + 1);
        av_ay = (float)(av_ay * m_cnt + abs(ay)) / (m_cnt + 1);
        av_az = (float)(av_az * m_cnt + abs(az - Z_ACC_CONST)) / (m_cnt + 1);

        m_cnt++;

        if (micros() - time > settings.runTime * 1000000)
          break;
      }

      motorStop();

      av_ax = trunc(av_ax);
      av_ay = trunc(av_ay);
      av_az = trunc(av_az);

      // Show result.
      y = showTitle("MEASURE COMPLETED");
      lcd.setCursor(0, y);
      lcd.print("X: ");

      String s = intToStr(prev_av_ax);
      lcd.print(s);
      lcd.print("  ");
      s = intToStr((int16_t)av_ax);
      lcd.print(s);

      y += (FONT_HEIGHT + 2);
      lcd.setCursor(0, y);
      lcd.print("Y: ");
      s = intToStr(prev_av_ay);
      lcd.print(s);
      lcd.print("  ");
      s = intToStr((int16_t)av_ay);
      lcd.print(s);

      y += (FONT_HEIGHT + 2);
      lcd.setCursor(0, y);
      lcd.print("Z: ");
      s = intToStr(prev_av_az);
      lcd.print(s);
      lcd.print("  ");
      s = intToStr((int16_t)av_az);
      lcd.print(s);

      y += (FONT_HEIGHT + 2);
      lcd.setCursor(0, y);
      lcd.print("Press OK to remeasure");
      y += (FONT_HEIGHT + 2);
      printCenter("L - exit, R - recal", y);
      lcd.display();

      // Store previouse result.
      prev_av_ax = av_ax;
      prev_av_ay = av_ay;
      prev_av_az = av_az;

      if ((btn = getButton()) == BtnOK)
        continue;
      
      break;
    }

    if (btn == BtnLeft)
      break;
  }
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// ESC
#define MOTOR_SPIN_UP_TIME  1

Servo esc;
bool  isRunning = false; // Motor state.

void motorStart()
{
  if (!isRunning)
  {
    ledOn();
    beep(TONE_START);

    // Start motor.
    esc.writeMicroseconds(settings.runPulse);
    // Wait for motor spin up.
    delay(MOTOR_SPIN_UP_TIME * 1000);

    isRunning = true;
  }
}

void motorStop()
{
  if (isRunning)
  {
    esc.writeMicroseconds(settings.minPulse);

    // Wait for motor stop.
    delay(MOTOR_SPIN_UP_TIME * 1000);

    beep(TONE_STOP);
    ledOff();

    isRunning = false;
  }
}

void calibrateESC()
{
  // Send max PWM first.
  uint8_t y = showTitle("ESC CALIBRATION");
  y += 4;
  y = printCenter("Unplug the battery", y);
  y = printCenter("from the ESC", y);
  y += 4;
  y = printCenter("Press OK when ready", y);
  lcd.display();

  ledOn();
  waitButton(BtnOK);

  esc.writeMicroseconds(MAX_PULSE_WIDTH);

  // Waiting for max throttle learning.
  y = showTitle("ESC CALIBRATION");
  y += 4;
  y = printCenter("Plug the battery and", y);
  y = printCenter("wait for ESC beep", y);
  y += 4;
  y = printCenter("Press OK when ready", y);
  lcd.display();

  ledOff();
  waitButton(BtnOK);

  // Waiting for min throttle learning.
  esc.writeMicroseconds(settings.minPulse);

  y = showTitle("ESC CALIBRATION");
  y += 4;
  y = printCenter("Wait for ESC", y);
  y = printCenter("READY BEEP", y);
  y += 4;
  y = printCenter("Press OK to EXIT", y);
  lcd.display();

  ledBlink();

  waitButton(BtnOK);
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// MAIN MENU
#define ITEM_VIBROMETER  1
#define ITEM_ESC_CALIB   2
#define ITEM_SETUP       3

// Shows and navigates through Main Menu. Return selected item.
uint8_t mainMenu()
{
  uint8_t selected = ITEM_VIBROMETER;

  while (1)
  {
    // Show menu.
    uint8_t y = showTitle("MAIN MENU");
    y = showMenuItem("Vibrometer", y, (selected == ITEM_VIBROMETER));
    y = showMenuItem("ESC Calib.", y, (selected == ITEM_ESC_CALIB));
    y = showMenuItem("Setup", y, (selected == ITEM_SETUP));
    lcd.display();

    // Navigation.
    switch (getButton())
    {
      case BtnLeft:
        selected--;
        break;

      case BtnOK:
        return selected;

      case BtnRight:
        selected++;
        break;
    }

    if (selected > ITEM_SETUP)
      selected = ITEM_VIBROMETER;
    else if (selected < ITEM_VIBROMETER)
      selected = ITEM_SETUP;
  }
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// SETUP MENU
#define ITEM_MIN      1
#define ITEM_RUN      2
#define ITEM_TIME     3
#define ITEM_SAVE     4
#define ITEM_CANCEL   5

#define PWM_CHANGE_VAL  20
#define PWM_MIN_VAL     800
#define PWM_MAX_VAL     1500
#define PWM_RUN_MAX_VAL 2500
#define TIME_CHANGE_VAL 1
#define TIME_MIN_VAL    5
#define TIME_MAX_VAL    10

// Show setup menu.
void showSetupMenu(uint8_t selected, bool change, uint16_t pwmMin, uint16_t pwmRun, uint8_t runTime)
{
  uint8_t y = showTitle("SETTINGS");
  //
  lcd.setCursor(0, y);
  if (selected == ITEM_MIN)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" Min pulse: ");
  if (selected == ITEM_MIN)
    lcd.setTextColor(WHITE);
  lcd.print("  ");
  if (selected == ITEM_MIN && change)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" ");
  lcd.print(pwmMin);
  lcd.print(" ");
  if (selected == ITEM_MIN && change)
    lcd.setTextColor(WHITE);

  y += (FONT_HEIGHT + 2);
  lcd.setCursor(0, y);
  if (selected == ITEM_RUN)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" Run pulse: ");
  if (selected == ITEM_RUN)
    lcd.setTextColor(WHITE);
  lcd.print("  ");
  if (selected == ITEM_RUN && change)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" ");
  lcd.print(pwmRun);
  lcd.print(" ");
  if (selected == ITEM_RUN && change)
    lcd.setTextColor(WHITE);

  y += (FONT_HEIGHT + 2);
  lcd.setCursor(0, y);
  if (selected == ITEM_TIME)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" Run  time: ");
  if (selected == ITEM_TIME)
    lcd.setTextColor(WHITE);
  lcd.print("  ");
  if (selected == ITEM_TIME && change)
    lcd.setTextColor(BLACK, WHITE);
  lcd.print(" ");
  lcd.print(runTime);
  lcd.print(" ");
  if (selected == ITEM_TIME && change)
    lcd.setTextColor(WHITE);

  y += (FONT_HEIGHT + 2);
  y = showMenuItem("Save", y, (selected == ITEM_SAVE));
  y = showMenuItem("Cancel", y, (selected == ITEM_CANCEL));

  lcd.display();
}

void setupMenu()
{
  uint16_t pwmMin = settings.minPulse;
  uint16_t pwmRun = settings.runPulse;
  uint8_t runTime = settings.runTime;

  bool change = false;

  uint8_t selected = ITEM_MIN;
  bool term = false;

  while (!term)
  {
    showSetupMenu(selected, change, pwmMin, pwmRun, runTime);

    switch (getButton())
    {
      case BtnLeft:
        if (change)
        {
          switch (selected)
          {
            case ITEM_MIN:
              pwmMin -= PWM_CHANGE_VAL;
              if (pwmMin < PWM_MIN_VAL)
                pwmMin = PWM_MIN_VAL;
              break;

            case ITEM_RUN:
              pwmRun -= PWM_CHANGE_VAL;
              if (pwmRun < pwmMin)
                pwmRun = pwmMin;
              break;

            case ITEM_TIME:
              runTime -= TIME_CHANGE_VAL;
              if (runTime < 1)
                runTime = 1;
              break;
          }
        }
        else
          selected--;
        break;

      case BtnOK:
        if (change)
          change = false;
        else
        {
          if (selected == ITEM_SAVE || selected == ITEM_CANCEL)
          {
            if (selected == ITEM_SAVE)
            {
              settings.minPulse = pwmMin;
              settings.runPulse = pwmRun;
              settings.runTime = runTime;

              EEPROM.put(SETTINGS_ADDR, settings);
            }
            term = true;
          }
          else
            change = true;
        }
        break;

      case BtnRight:
        if (change)
        {
          switch (selected)
          {
            case ITEM_MIN:
              pwmMin += PWM_CHANGE_VAL;
              if (pwmMin > PWM_MAX_VAL)
                pwmMin = PWM_MAX_VAL;
              break;

            case ITEM_RUN:
              pwmRun += PWM_CHANGE_VAL;
              if (pwmRun > PWM_RUN_MAX_VAL)
                pwmRun = PWM_RUN_MAX_VAL;
              break;

            case ITEM_TIME:
              runTime += TIME_CHANGE_VAL;
              if (runTime > TIME_MAX_VAL)
                runTime = TIME_MAX_VAL;
          }
        }
        else
          selected++;
        break;
    }

    if (selected < ITEM_MIN)
      selected = ITEM_CANCEL;
    else if (selected > ITEM_CANCEL)
      selected = ITEM_MIN;
  }
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// LOGO DATA
const uint8_t logo [] PROGMEM = {
  0x00, 0x00, 0x00, 0x1F, 0xFC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x03, 0x03, 0x80, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x08, 0x02, 0x40, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x40, 0x03, 0xC0, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x80, 0x03, 0xC0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x02, 0x00, 0x03, 0x40, 0x00, 0x40, 0x00, 0x31, 0xA7, 0x9E, 0x7C, 0xC3, 0x3F, 0xEF, 0x78,
  0x00, 0x04, 0x00, 0x02, 0x40, 0x00, 0x20, 0x00, 0x11, 0x24, 0x91, 0xC6, 0xE3, 0x20, 0x88, 0x44,
  0x00, 0x10, 0x00, 0x06, 0x00, 0x00, 0x08, 0x00, 0x11, 0x24, 0x91, 0x82, 0xA5, 0x20, 0x88, 0x44,
  0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x1B, 0x27, 0x9E, 0x82, 0xA5, 0x3C, 0x8F, 0x78,
  0x00, 0x40, 0x00, 0x06, 0x20, 0x00, 0x02, 0x00, 0x0A, 0x24, 0x92, 0x82, 0xBD, 0x20, 0x88, 0x48,
  0x00, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x0A, 0x24, 0x93, 0xC6, 0x99, 0x20, 0x88, 0x4C,
  0x01, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x80, 0x0C, 0x27, 0x11, 0x7C, 0x99, 0x3C, 0x8F, 0x44,
  0x00, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x02, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x04, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x08, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x10, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x04, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x20, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x06, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x02, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x03, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x07, 0xF0, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x0B, 0xF0, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x0B, 0xC8, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x1F, 0x20, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x2E, 0x7C, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x12, 0xA0, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x0C, 0x34, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x12, 0xC0, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x6C, 0x36, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x1E, 0xC0, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x00, 0x76, 0x6E, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x03, 0x33, 0xCF, 0x80, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x04, 0x39, 0x1E, 0xE0, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0x30, 0x3B, 0xFC, 0x1C, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x80, 0x00, 0xC0, 0x6C, 0x0C, 0x0E, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x01, 0x01, 0xC0, 0x03, 0x03, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x06, 0x03, 0x80, 0x01, 0x80, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0x18, 0x0E, 0x00, 0x00, 0x60, 0x70, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x40, 0x30, 0x38, 0x00, 0x00, 0x30, 0x1C, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x40, 0xE0, 0x00, 0x00, 0x0C, 0x0E, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x21, 0x81, 0xC0, 0x00, 0x00, 0x03, 0x07, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x03, 0x0F, 0x00, 0x00, 0x00, 0x00, 0xC1, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x07, 0xBC, 0x00, 0x00, 0x00, 0x00, 0x71, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x15, 0xE0, 0x00, 0x00, 0x00, 0x00, 0x0F, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x0F, 0x80, 0x00, 0x00, 0x00, 0x00, 0x03, 0xF0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x04, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0E, 0xBF, 0x95, 0xE5, 0x76, 0x33, 0x1F, 0x80,
  0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x04, 0xA5, 0xA7, 0xED, 0xF4, 0x44, 0x92, 0x80,
  0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x04, 0xA5, 0x66, 0x1E, 0x84, 0x44, 0x92, 0x80,
  0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x06, 0xA5, 0x65, 0xD2, 0x74, 0xF3, 0x12, 0x80,
  0x00, 0x00, 0x80, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x40, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x08, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x03, 0x00, 0x00, 0xC0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x3F, 0xF8, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// INITIALIZATION.
void setup()
{
  // First read Vibrometer settings as we need it to initialize ESC.
  EEPROM.get(SETTINGS_ADDR, settings);
  // Make sure we read correct data.
  if (settings.minPulse == 0xFFFF || settings.minPulse == 0)
    settings.minPulse = 1000;
  if (settings.runPulse == 0xFFFF || settings.runPulse == 0)
    settings.runPulse = 1200;
  if (settings.runTime == 0xFF || settings.runTime == 0)
    settings.runTime = 5;

  // Initialize LED.
  pinMode(LED_PIN, OUTPUT);
  digitalWrite(LED_PIN, LOW); // Current state is LOW as well.

  // Init BEEPER PIN.
  pinMode(BEEPER_PIN, OUTPUT);
  digitalWrite(BEEPER_PIN, LOW); // No sound

  // Initialize button debouncers.
  pinMode(BUTTON_LEFT_PIN, INPUT);
  btnLeft.attach(BUTTON_LEFT_PIN);
  btnLeft.interval(BUTTON_DEBOUNCE_INTERVAL);

  pinMode(BUTTON_OK_PIN, INPUT);
  btnOk.attach(BUTTON_OK_PIN);
  btnOk.interval(BUTTON_DEBOUNCE_INTERVAL);

  pinMode(BUTTON_RIGHT_PIN, INPUT);
  btnRight.attach(BUTTON_RIGHT_PIN);
  btnRight.interval(BUTTON_DEBOUNCE_INTERVAL);

  // Initialize LCD.
  lcd.begin(SSD1306_SWITCHCAPVCC, LCD_I2C_ADDR);
  // Show logo for 2 seconds.
  lcd.clearDisplay();
  lcd.drawBitmap(0, 0, &logo[0], SSD1306_LCDWIDTH, SSD1306_LCDHEIGHT, WHITE);
  lcd.display();
  delay(4000);

  // Initialize MPU.
  TWBR = 24;
  mpu.initialize();

  // Initialize ESC.
  esc.attach(ESC_PIN, settings.minPulse, MAX_PULSE_WIDTH);
  esc.writeMicroseconds(settings.minPulse);
}
//--------------------------------------------------------------------------------------


//--------------------------------------------------------------------------------------
// MAIN LOOP.
void loop()
{
  switch (mainMenu())
  {
    case ITEM_VIBROMETER:
      measure();
      break;

    case ITEM_ESC_CALIB:
      calibrateESC();
      break;

    case ITEM_SETUP:
      setupMenu();
      break;
  }
}
//--------------------------------------------------------------------------------------
